@@ -1,12 +1,13 @@
-#!/usr/bin/env python
+#! /usr/bin/env python
+
 
 '''
 dotfile管理工具(dotfiles manager)，dotfile指保存配置信息的文件或包含配置文件的文件夹
 
 Usage:
-    dfm add <install_path>
+    dfm add <install_path> [--system]
     dfm rm <path>
-    dfm install [<install_path>]
+    dfm install [<save_path>]
     dfm share <save_path> <install_path>
 
 Commands:
@@ -21,7 +22,8 @@ Arguments:
     install_path dotfile的安装路径
 
 Options:
-    -h --help
+    -h --help  显示帮助
+    --system  该dotfile和操作系统相关
 '''
 
 from docopt import docopt
@@ -31,9 +33,11 @@ import os
 import platform
 import shutil
 import hashlib
+import posixpath
+import ctypes
 
 
-dotfiles_root = os.path.join(*[Path.home(), "dotfiles_test"])
+dotfiles_root = os.path.join(*[Path.home(), "dotfiles"])
 
 
 def __load_config():
@@ -60,7 +64,20 @@ def __get_os_name():
 def __normalize_path(path):
     if path is None:
         return None
-    return os.path.abspath(os.path.normpath(path))
+    return os.path.abspath(os.path.normpath(__expanduser(path)))
+
+
+def __expanduser(path):
+    if path is None:
+        return None
+
+    if __get_os_name() != "windows":
+        return os.path.expanduser(path)
+
+    home = str(Path.home())
+    if path.startswith('~'):
+        path = path.replace('~', home, 1)
+    return path
 
 
 # opposite of os.path.expanduser
@@ -74,14 +91,15 @@ def __shrinkuser(path):
     return path
 
 
-def __get_save_path(install_path):
+def __get_save_path(install_path, system):
     install_path = __shrinkuser(install_path)
     md5 = hashlib.md5()
     md5.update(str(os.path.dirname(install_path)).encode("utf8"))
     save_dir = md5.hexdigest()
 
     filename = os.path.basename(install_path)
-    save_path = os.path.join(*[dotfiles_root, save_dir, filename])
+    system_sep = __get_os_name() if system else ""
+    save_path = os.path.join(*[dotfiles_root, save_dir, system_sep, filename])
     return save_path
 
 
@@ -119,16 +137,17 @@ def __get_path(config, rel_save_path):
     if os_name not in config["dotfiles"][rel_save_path]:
         return None
     install_path = config["dotfiles"][rel_save_path][os_name]["path"]
-    return os.path.expanduser(install_path)
+    return __expanduser(install_path)
 
 
-def __add(install_path, config):
-    abs_save_path = __get_save_path(install_path)
+def __add(install_path, system, config):
+    abs_save_path = __get_save_path(install_path, system)
     os.makedirs(os.path.dirname(abs_save_path), exist_ok=True)
     shutil.move(install_path, abs_save_path)
     os.symlink(abs_save_path, install_path)
 
     rel_save_path = os.path.relpath(abs_save_path, dotfiles_root)
+    rel_save_path = rel_save_path.replace(os.sep, posixpath.sep)
     print("Add %s to %s" % (install_path, rel_save_path))
     return __set_path(config, rel_save_path, install_path)
 
@@ -138,6 +157,7 @@ def __rm(path, config):
     if os.path.islink(path):
         abs_save_path = os.readlink(path)
     rel_save_path = os.path.relpath(abs_save_path, dotfiles_root)
+    rel_save_path = rel_save_path.replace(os.sep, posixpath.sep)
 
     install_path = __get_path(config, rel_save_path)
     if install_path is None:
@@ -167,6 +187,7 @@ def __install(abs_save_path, config):
     rel_save_path = None
     if abs_save_path is not None:
         rel_save_path = os.path.relpath(abs_save_path, dotfiles_root)
+        rel_save_path = rel_save_path.replace(os.sep, posixpath.sep)
         if __get_path(config, rel_save_path) is None:
             print("%s is not kept in dotfiles" % rel_save_path)
             return config
@@ -179,6 +200,7 @@ def __install(abs_save_path, config):
         if item_install_path is None:
             continue
         item_abs_save_path = os.path.join(dotfiles_root, item_rel_save_path)
+        item_abs_save_path = item_abs_save_path.replace(posixpath.sep, os.sep)
 
         if __mklink(item_abs_save_path, item_install_path):
             print("Install %s -> %s" % (item_rel_save_path, item_install_path))
@@ -187,6 +209,7 @@ def __install(abs_save_path, config):
 
 def __share(abs_save_path, install_path, config):
     rel_save_path = os.path.relpath(abs_save_path, dotfiles_root)
+    rel_save_path = rel_save_path.replace(os.sep, posixpath.sep)
     if rel_save_path not in config["dotfiles"]:
         print("%s is not kept in dotfiles" % rel_save_path)
         return config
@@ -201,6 +224,7 @@ def __share(abs_save_path, install_path, config):
 def __dispatch(args):
     def check_add_args():
         path = __normalize_path((args["<install_path>"]))
+        system = args["--system"]
         if not os.path.isfile(path) and not os.path.isdir(path):
             print("%s is not valid file or directory" % path)
             exit(-1)
@@ -210,7 +234,7 @@ def __dispatch(args):
         if not path.startswith(str(Path.home())):
             print("%s must be in home" % path)
             exit(-1)
-        if os.path.exists(__get_save_path(path)):
+        if os.path.exists(__get_save_path(path, system)):
             print("%s has been kept in dotfiles" % path)
             exit(-1)
 
@@ -227,7 +251,8 @@ def __dispatch(args):
         check_add_args()
 
         def add(config):
-            return __add(__normalize_path(args["<install_path>"]), config)
+            return __add(__normalize_path(args["<install_path>"]),
+                         args["--system"], config)
         return add
     elif args["rm"]:
         check_rm_args()
@@ -249,6 +274,11 @@ def __dispatch(args):
 
 
 def main():
+    if __get_os_name() == "windows":
+        if ctypes.windll.shell32.IsUserAnAdmin() == 0:
+            print("dfm must run with Administrator priviledges under Windows")
+            return
+
     args = docopt(__doc__)
 
     config = __load_config()
